#include "ctcp.h"
#include "global.h"
#include "message.h"
#include "window.h"
#include "markup.h"

using namespace IRC;

bool CTCPMessage::isCTCPMessage( const QString &text )
{
    if ( text.length() <= 2 )
        return false;

    return ( text[ 0 ] == QChar( CTCPMark ) && text[ text.length() - 1 ] == QChar( CTCPMark ) );
}

CTCPMessage CTCPMessage::fromString( const QString &text )
{
    // ### really waste extra cycles for that?
    if ( !isCTCPMessage( text ) ) 
        return CTCPMessage();

    int typeLength = text.find( ' ' );
    const uchar typeStartIndex = 1;

    if ( typeLength == -1 )
        typeLength = text.length() - 2;
    else
        typeLength--;

    const uint parameterStartIdx = typeStartIndex + typeLength + 1; 
    const uint parameterLength = text.length() - parameterStartIdx - 1; // no +1 because of the trailing \01

    CTCPMessage result;
    result.m_type = text.mid( typeStartIndex, typeLength );
    result.m_parameters = QStringList::split( ' ', text.mid( parameterStartIdx, parameterLength ) );

    return result;
}

// don't think about changing the return type to a const reference!
QStringList CTCPMessage::parameters( QuotingFlag quoting ) const
{
    if ( quoting == Quoted )
        return m_parameters;

    QStringList result;

    for ( QStringList::ConstIterator it = m_parameters.begin();
          it != m_parameters.end(); ++it )
        result << dequoteString( *it, quoting );

    return result;
}

QString CTCPMessage::parameter( uint index, QuotingFlag quoting ) const
{
    if ( m_parameters.count() < index )
        return QString::null;

    return dequoteString( m_parameters[ index ], quoting );
}

QString CTCPMessage::dequoteString( const QString &input, QuotingFlag quoting )
{
    // ### implement me
    return input;
}

QString CTCPAction::text( const Message &context ) const
{
    Person person = context.sender();

    return Markup::tagAction( person.nick() + ' ' + Markup::tagColorCodes( rawText() ) );
}

CTCPHandler::CTCPHandler( QObject *parent, const char *name )
    : QObject( parent, name )
{
}

void CTCPHandler::handleCTCPMessage( const QString &input, const IRC::Message &context, IRC::Window *window )
{
    CTCPMessage ctcpMsg = CTCPMessage::fromString( input );
    if ( !ctcpMsg.isValid() )
        return;

    // ###
    if ( ctcpMsg.type() == CTCPAction::name() ) {
        handleCTCPAction( ctcpMsg, context, window );
        return;
    }

    // ### error handling
}

void CTCPHandler::handleCTCPAction( const CTCPAction &action, const IRC::Message &context, IRC::Window *window )
{
    // #### Markup::tag..Action
    window->printMessage( action.text( context ) );
}

bool CTCPMessageFactory::registerMessageType( const QString &typeName, CTCPMessageFactoryFunction factoryFunc )
{
    // second contains whether the insertion worked or not
    return m_typeMap.insert( FactoryMap::value_type( typeName, factoryFunc ) ).second;
}

CTCPMessage *CTCPMessageFactory::createMessage( const QString &typeName, const CTCPMessage &body )
{
    FactoryMap::ConstIterator it = m_typeMap.find( typeName );
    if ( it == m_typeMap.end() )
        return 0;

    return ( *it )( body );
}

#include "ctcp.moc"

/* vim: et sw=4 ts=4
 */
